# GDB Remote Debugging

## Install GDB Server on Guest

Run `sudo apk add gdb` on phone.

## Install GDB Multiarch on Host

To debug ARMv7 you need **multiarch** variant of GDB.

Ubuntu: `sudo apt install gdb-multiarch`

Then run `gdb-multiarch` executable.

## Connect to GDB Server

### Start server on Guest

#### Run new program

Run `gdbserver` on a forwarded TCP port:

`gdbserver :port <program> [args]`

Example: `gdbserver :5000 lxqt-panel -c panel.conf`

#### Attach to running program

`gdbserver --attach :port <pid>`

Example: `gdbserver --attach :5000 $(pidof lxqt-panel)`

### Connect from Host

Run `gdb-multiarch`

Then inside GDB shell type: `target remote localhost:5001`

Where 5001 is the example local ssh tunnel end.

## Source mapping

### Sysroot

First tell GDB to get symbols from a local copy of the target program stored on host. This avoids having it send symbols over SSH which is slow.

Run `set sysroot /path/to/local/folder/usr/bin`

**TIP:** pmbootstrap already stores a copy of device rootfs.

Usually in: `~/.local/var/pmbootstrap/chroot_rootfs_samsung-serranolte/` (replace device name).

You can use it as GDB sysroot, but remember to reflect changes made on device.

So when you install target executable on device, you must install same version also on local rootfs chroot.

You can open a chroot shell with `pmbootstrap chroot -r` and inside it run apk with `apk add <package name>`

**NOTE:** If target executable on device and local copy do not match, GDB will complain and may also crash. To be sure do a `sha512sum` or read executable **Build ID**.

### Substitute paths

Now tell GDB to map hardcoded paths in target executable to your source folder.

**NOTE:** hardcoded paths refer to where package is actually built. Usually it's pmbootstrap chroot buildroot.  
You can get these path by typing `info sources` inside GDB.

```
set substitute-path /home/pmos/build/src/<package-name> /path/to/git/cloned/<package-name>
```


## Build ID

It's an hash string burned inside executable, GDB uses it to check local version and remote executable match.

You can read this info with: `readelf -n /path/to/executable`

Example:

```
readelf -n ./lxqt-mobile-appmenu 

Displaying notes found in: .note.gnu.build-id
  Owner                Data size        Description
  GNU                  0x00000014       NT_GNU_BUILD_ID (unique build ID bitstring)
    Build ID: a4465a57db750b04db988d5ca1fb93ef7eb5c5cd
```